@@ -45,7 +45,7 @@ ProfileManager::~ProfileManager()
 
 // Login, calls StorageAgent to retrieve the user
 // profile data
-bool ProfileManager::Login(int userID)
+gboolean ProfileManager::Login(guint32 userID)
 {
 	// Store the id
 	iUser = userID;
@@ -76,7 +76,7 @@ bool ProfileManager::Login(int userID)
 }
 
 // Logout
-bool ProfileManager::Logout()
+gboolean ProfileManager::Logout()
 {
 
 	// Store the user data
@@ -108,7 +108,7 @@ string ProfileManager::ViewDetail(string& detail)
 
 // Edit user details. Takes the name of the detail and the new Value
 // as a string
-bool ProfileManager::EditDetails(string& detail, string& newValue)
+gboolean ProfileManager::EditDetails(string& detail, string& newValue)
 {
 	// If the detail changing operation is failure, function return false
 	return iStorageAgent->setDetails(detail, newValue);
@@ -116,7 +116,7 @@ bool ProfileManager::EditDetails(string& detail, string& newValue)
 
 // This function is used to Force another user in to the
 // profile manager. Requires admin level in the profile
-bool ProfileManager::ForceUser(int userID)
+gboolean ProfileManager::ForceUser(guint32 userID)
 {
 	string auth;
 	// Get detail authLevel from the profile
@@ -131,11 +131,16 @@ bool ProfileManager::ForceUser(int userID)
 		if (Logout() == true)
 		{
 			Login(userID);
-			return true;
+			return TRUE;
 		}
-		else return false;
+		else return FALSE;
 	}
-	else return false;
+	else return FALSE;
+}
+
+guint32 ProfileManager::GetId()
+{
+	return iUser;
 }
 
 void log_wrap(char* msg, int info)
@@ -145,118 +150,121 @@ void log_wrap(char* msg, int info)
 }
 
 // New manager object
-ProfileManager* gems_profilemanager_new_manager()
+ProfileManager* profilemanager_new_manager()
 {
 	return ProfileManager::GetInstance();
 }
 
 // Delete manager object
-void gems_profilemanager_delete_manager(ProfileManager* _aManager)
+void profilemanager_delete_manager(ProfileManager* _aManager)
 {
 	delete _aManager;
 }
 
 // Login to manager (no auth checking yet)
-int gems_profilemanager_login(ProfileManager* _aManager, int _userId)
+gboolean profilemanager_login(ProfileManager* _aManager, guint32 _userId)
 {
-	// When true, return 1
-	return (_aManager->Login(_userId)) ? 1 : 0;
+	return _aManager->Login(_userId);
 }
 
 // Logout from manager
-int gems_profilemanager_logout(ProfileManager* _aManager)
+gboolean profilemanager_logout(ProfileManager* _aManager)
 {
-	// When true return 1
-	return (_aManager->Logout()) ? 1 : 0;
+	return _aManager->Logout();
 }
 
 // View one information
-const char* gems_profilemanager_view_info(ProfileManager* _aManager, char* _parameter)
+const gchar* profilemanager_view_info(ProfileManager* _aManager, gchar* _parameter)
 {
 	string param = string(_parameter);
 	return (_aManager->ViewDetail(param)).c_str();
 }
 
 // Edit own info
-int gems_profilemanager_edit_info(ProfileManager* _aManager, char* _parameter, char* _value)
+gboolean profilemanager_edit_info(ProfileManager* _aManager, gchar* _parameter, gchar* _value)
 {
 	string param = string(_parameter);
 	string value = string(_value);
-	return (_aManager->EditDetails(param,value)) ? 1 : 0;
+	return _aManager->EditDetails(param,value);
+}
+
+guint32 profilemanager_get_userid(ProfileManager* _aManager)
+{
+	return _aManager->GetId();
 }
 
 // Get own username
-const char* gems_profilemanager_get_username(ProfileManager* _aManager)
+const gchar* profilemanager_get_username(ProfileManager* _aManager)
 {
 	string param = string(PROF_PARAM_USERNAME);
 	return (_aManager->ViewDetail(param)).c_str();
 }
 
 // Get first name
-const char* gems_profilemanager_get_firstname(ProfileManager* _aManager)
+const gchar* profilemanager_get_firstname(ProfileManager* _aManager)
 {
 	string param = string(PROF_PARAM_FIRSTNAME);
 	return (_aManager->ViewDetail(param)).c_str();
 }
 
 // Get last name
-const char* gems_profilemanager_get_lastname(ProfileManager* _aManager)
+const gchar* profilemanager_get_lastname(ProfileManager* _aManager)
 {
 	string param = string(PROF_PARAM_LASTNAME);
 	return (_aManager->ViewDetail(param)).c_str();
 }
 
 // Get age
-const char* gems_profilemanager_get_age(ProfileManager* _aManager)
+guint16 profilemanager_get_age(ProfileManager* _aManager)
 {
 	string param = string(PROF_PARAM_AGE);
-	return (_aManager->ViewDetail(param)).c_str();
+	return atoi((_aManager->ViewDetail(param)).c_str());
 }
 
-unsigned int gems_profilemanager_get_points(ProfileManager* _aManager)
+guint32 profilemanager_get_points(ProfileManager* _aManager)
 {
 	string param = string(PROF_PARAM_POINTS);
-	return (unsigned int)atoi(_aManager->ViewDetail(param).c_str());
+	return (guint32)atoi(_aManager->ViewDetail(param).c_str());
 }
 
-int gems_profilemanager_set_username(ProfileManager* _aManager, char* _value)
+gboolean profilemanager_set_username(ProfileManager* _aManager, gchar* _value)
 {
 	string param = string(PROF_PARAM_USERNAME);
 	string value = string(_value);
-	return (_aManager->EditDetails(param,value)) ? 1 : 0;
+	return _aManager->EditDetails(param,value);
 }
 
-int gems_profilemanager_set_firstname(ProfileManager* _aManager, char* _value)
+gboolean profilemanager_set_firstname(ProfileManager* _aManager, gchar* _value)
 {
 	string param = string(PROF_PARAM_FIRSTNAME);
 	string value = string(_value);
-	return (_aManager->EditDetails(param,value)) ? 1 : 0;
+	return _aManager->EditDetails(param,value);
 }
 
-int gems_profilemanager_set_lastname(ProfileManager* _aManager, char* _value)
+gboolean profilemanager_set_lastname(ProfileManager* _aManager, gchar* _value)
 {
 	string param = string(PROF_PARAM_LASTNAME);
 	string value = string(_value);
-	return (_aManager->EditDetails(param,value)) ? 1 : 0;
+	return _aManager->EditDetails(param,value);
 }
 
-int gems_profilemanager_set_age(ProfileManager* _aManager, unsigned int _value)
+gboolean profilemanager_set_age(ProfileManager* _aManager, guint16 _value)
 {
 	stringstream strvalue;
 	string param = string(PROF_PARAM_AGE);
 	
-	if(_value > 120) return -1;
+	if(_value > 120) return FALSE;
 	
 	strvalue << _value;
 	string value = strvalue.str();
 	
-	return (_aManager->EditDetails(param,value)) ? 1 : 0;
+	return _aManager->EditDetails(param,value);
 }
 
-int gems_profilemanager_add_points(ProfileManager* _aManager, unsigned int _value)
+gboolean profilemanager_add_points(ProfileManager* _aManager, guint32 _value)
 {
 	string param = string(PROF_PARAM_POINTS);
-	int points = atoi((_aManager->ViewDetail(param)).c_str());
+	guint32 points = atoi((_aManager->ViewDetail(param)).c_str());
 	
 	points += _value;
 	
@@ -265,26 +273,26 @@ int gems_profilemanager_add_points(ProfileManager* _aManager, unsigned int _valu
 	
 	string value = strvalue.str();
 	
-	return (_aManager->EditDetails(param,value)) ? 1 : 0;
+	return _aManager->EditDetails(param,value);
 }
 
-int gems_profilemanager_remove_points(ProfileManager* _aManager, unsigned int _value)
+gboolean profilemanager_remove_points(ProfileManager* _aManager, guint32 _value)
 {
 	string param = string(PROF_PARAM_POINTS);
-	int points = atoi((_aManager->ViewDetail(param)).c_str());
+	guint32 points = atoi((_aManager->ViewDetail(param)).c_str());
 	
-	if((points - (int)_value) < 0) return -1;
-	points = points - _value;
+	if((points - _value) < 0) points = 0;
+	else points = points - _value;
 	
 	stringstream strvalue;
 	strvalue << points;
 	
 	string value = strvalue.str();
 	
-	return (_aManager->EditDetails(param,value)) ? 1 : 0;	
+	return _aManager->EditDetails(param,value);	
 }
 
-void gems_profilemanager_reset_points(ProfileManager* _aManager)
+void profilemanager_reset_points(ProfileManager* _aManager)
 {
 	string param = string(PROF_PARAM_POINTS);
 	string value = "0";
