@@ -1,17 +1,7 @@
 #ifndef __PM_STORAGEAGENT_H_
 #define __PM_STORAGEAGENT_H_
 
-#define JAMMOPROFILE ".jammo-profiles/"
-#define PROFILETYPE ".csv"
-
-// Profile parameters
-#define PROF_PARAM_USERID "userID"
-#define PROF_PARAM_USERNAME "username"
-#define PROF_PARAM_AUTHLEVEL "authLevel"
-#define PROF_PARAM_FIRSTNAME "firstName"
-#define PROF_PARAM_LASTNAME "lastName"
-#define PROF_PARAM_POINTS "gamePoints"
-#define PROF_PARAM_AGE "age"
+#include "gems_definitions.h"
 
 #ifdef __cplusplus
 
@@ -19,6 +9,7 @@
 #include <iostream>
 #include <vector>
 #include <iterator>
+#include <glib.h>
 
 using namespace std;
 
@@ -32,14 +23,14 @@ public:
 	
 	// Functions
 	// Synchronize the data
-	bool Synchronize();	
+	gboolean Synchronize();	
 	
 	// Retrieves the user data from the database
 	// currently just creates static data
-	bool Retrieve(int userID); 
+	gboolean Retrieve(guint32 userID); 
 	
 	// Stores the user profile data to database
-	bool Store(int userID);
+	gboolean Store(guint32 userID);
 	
 	// Gets the details 
 	string getDetails();
@@ -48,10 +39,10 @@ public:
 	string getDetail(string& detail);
 	
 	// Set a detail
-	bool setDetails(string& detail, string& newvalue);
+	gboolean setDetails(string& detail, string& newvalue);
 	
 	// Clear all the user details
-	bool clearDetails();
+	gboolean clearDetails();
 	
 private:
 	// The map is for storing key-value pairs of user
