@@ -31,13 +31,36 @@ License:LGPL 2.1
 
 #define M_PI_M2 ( M_PI + M_PI )
 
+// time that is the threshold for discarding motion events
+// there is no need to store all motion events that are received really closely
+#define DISCARDTIME 1000000
+
+// minimum frequency to slide between samples if sliding is needed
+#define MINSLIDE 0.002
+
+// multiplier for difference of target and current frequency between samples
+#define DISTANCEFACTOR 0.00076
+
+// sample count for rounding
+#define ROUNDINGCOUNT 10000
+
+// volume of blow sound with FM_MODULATION algorithm
+#define BLOW_VOLUME 0.25
+
+/*	This is temporary hack:
+*/
+#define DATA_DIR "/opt/jammo"
+/* Actually, DATA_DIR should be defined using configure script... */
+
+#define SLIDER_FOLDER DATA_DIR "/virtual-instruments/slider/"
+
 static const GstElementDetails gst_jammo_slider_details =
 GST_ELEMENT_DETAILS ("JamMo Slider",
     "Source/Audio",
     "JamMo Slider",
     "Aapo Rantalainen <aapo.rantalainen@gmail.com>, Mikko Gynther <mikko.gynther@lut.fi>");
 
-#define DEFAULT_SAMPLES_PER_BUFFER   1024
+#define DEFAULT_SAMPLES_PER_BUFFER   256
 #define DEFAULT_INSTRUMENT           0
 #define DEFAULT_VOLUME               0.6
 #define DEFAULT_IS_LIVE              FALSE
@@ -57,8 +80,9 @@ enum
   PROP_0,
   PROP_SAMPLES_PER_BUFFER,
   PROP_INSTRUMENT,
-  PROP_SLIDER_FREQ,
-  PROP_SLIDER_STATE,
+  PROP_ADD_EVENT,
+  PROP_REMOVE_EVENT,
+  PROP_CLEAR_EVENTS,
   PROP_ATTACK,
   PROP_DECAY,
   PROP_SUSTAIN,
@@ -68,6 +92,8 @@ enum
   PROP_TIMESTAMP_OFFSET,
   PROP_CAN_ACTIVATE_PUSH,
   PROP_CAN_ACTIVATE_PULL,
+	PROP_RECORDING,
+	PROP_EVENTLIST,
   PROP_LAST
 };
 
@@ -162,7 +188,7 @@ short    n=0;
 short karplus_strong (short x, gfloat length) {
   long            a, b;       /* No need to remember a and b. */
 
-  a = (long)x + interpolation(length-(int)length, y[(int)(n - length) & BITMASK],y[(gint)(n - (length+1)) & BITMASK]); /* interpolation */
+  a = (long)x + interpolation(length-(int)length, y[((gint)(n - length)) & BITMASK],y[((gint)(n - (length+1))) & BITMASK]); /* interpolation */
 
   //b = ((a + ap) * 127) >> 8;            /* Coeff = +0.49609375. */
     b = ((a + ap) * 127) / (254+length*0.01); //higher coeff on lower tones
@@ -173,7 +199,7 @@ short karplus_strong (short x, gfloat length) {
 
 
 gfloat karplus(GstJammoSlider * src){
-  gfloat freq = src->slider_freq_slow;
+  gfloat freq = src->freq_slow;
   short inp;
   if (src->adsr_counter < src->samplerate/freq)  /* white noise BURST. */
       inp = (short)((rand() & 32767) - 16384);
@@ -211,10 +237,20 @@ static void gst_jammo_slider_class_init (GstJammoSliderClass * klass)
           "What instrument is used",
           0, NUMBER_OF_INSTRUMENTS-1, DEFAULT_INSTRUMENT,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
-  g_object_class_install_property (gobject_class, PROP_SLIDER_FREQ,
-      g_param_spec_float ("slider-freq", "Freq of slider", "Freq of slider",
-          32.0,4000,440,
-          G_PARAM_READWRITE | GST_PARAM_CONTROLLABLE | G_PARAM_STATIC_STRINGS ));
+  g_object_class_install_property(gobject_class, PROP_REMOVE_EVENT,
+                                 g_param_spec_pointer ("remove-event",
+                                 "Removes both start and stop events",
+                                 "",
+                                 G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK |G_PARAM_STATIC_BLURB));
+  g_object_class_install_property(gobject_class, PROP_ADD_EVENT,
+                                 g_param_spec_pointer ("add-event",
+                                 "Add event",
+                                 "",
+                                 G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK |G_PARAM_STATIC_BLURB));
+  g_object_class_install_property (gobject_class, PROP_CLEAR_EVENTS,
+      g_param_spec_boolean ("clear-events", "Clear events",
+          "Clear event list", FALSE,
+          G_PARAM_WRITABLE | G_PARAM_STATIC_STRINGS));
  g_object_class_install_property (gobject_class, PROP_ATTACK,
       g_param_spec_int ("attack", "attack",
           "attack",
@@ -234,10 +270,6 @@ static void gst_jammo_slider_class_init (GstJammoSliderClass * klass)
       g_param_spec_float ("sustain", "sustain", "sustain",
           0.0,1.0,DEFAULT_SUSTAIN,
           G_PARAM_READWRITE | GST_PARAM_CONTROLLABLE | G_PARAM_STATIC_STRINGS ));
- g_object_class_install_property (gobject_class, PROP_SLIDER_STATE,
-      g_param_spec_boolean ("slider-state", "State of slider", "State of slider",
-          FALSE,
-          G_PARAM_READWRITE | GST_PARAM_CONTROLLABLE | G_PARAM_STATIC_STRINGS ));  
   g_object_class_install_property (gobject_class, PROP_VOLUME,
       g_param_spec_double ("volume", "Volume", "Volume of test signal", 0.0,
           1.0, DEFAULT_VOLUME,
@@ -260,7 +292,13 @@ static void gst_jammo_slider_class_init (GstJammoSliderClass * klass)
       g_param_spec_boolean ("can-activate-pull", "Can activate pull",
           "Can activate in pull mode", DEFAULT_CAN_ACTIVATE_PULL,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
-
+  g_object_class_install_property (gobject_class, PROP_RECORDING,
+      g_param_spec_boolean ("recording", "Recording",
+          "Recording mode", TRUE,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+  g_object_class_install_property (gobject_class, PROP_EVENTLIST,
+      g_param_spec_pointer ("eventlist", "A list of events",
+          "Get or set a list of events", G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   gstbasesrc_class->set_caps = GST_DEBUG_FUNCPTR (gst_jammo_slider_setcaps);
   gstbasesrc_class->is_seekable =
       GST_DEBUG_FUNCPTR (gst_jammo_slider_is_seekable);
@@ -285,14 +323,19 @@ static void gst_jammo_slider_init (GstJammoSlider * src, GstJammoSliderClass * g
   src->format = GST_JAMMO_SLIDER_FORMAT_NONE;
 
   src->volume = DEFAULT_VOLUME;
-  src->slider_rounding_counter=0;
-  src->slider_first_freq=TRUE;
+  src->rounding_counter=0;
+
+	src->last_event_processed=FALSE;
+	src->time=0;
 
   src->attack  = DEFAULT_ATTACK;
   src->decay   = DEFAULT_DECAY;
   src->sustain = DEFAULT_SUSTAIN;
   src->release = DEFAULT_RELEASE;
 
+	// by default slider records
+	src->recording=TRUE;
+
   /* we operate in time */
   gst_base_src_set_format (GST_BASE_SRC (src), GST_FORMAT_TIME);
   gst_base_src_set_live (GST_BASE_SRC (src), DEFAULT_IS_LIVE);
@@ -306,6 +349,34 @@ static void gst_jammo_slider_init (GstJammoSlider * src, GstJammoSliderClass * g
 // This works without this.
 //  gst_base_src_set_blocksize (GST_BASE_SRC (src), -1)
 
+	// open blow sound file for flute like slider
+	char * path=NULL;
+	path=g_strdup_printf("%s%s",SLIDER_FOLDER,"slider_blow.wav");
+	printf("opening file %s\n", path);
+	src->blow_file = sf_open(path, SFM_READ, &(src->blow_file_info));
+	if (src->blow_file==NULL) {
+		printf("opening failed\n");
+	}
+	else {
+		// get number of frames
+		guint length = sf_seek(src->blow_file, 0, SEEK_END);
+		sf_seek(src->blow_file, 0, SEEK_SET);
+	
+		// allocate buffer
+		src->blow_table = malloc(sizeof(gint16)*length);
+	
+		// read to buffer
+		gint index=0;
+		while (sf_read_short(src->blow_file, &(src->blow_table[index]), 512)==512) {
+			index+=512;
+		}
+		sf_close(src->blow_file);
+	}
+	
+	src->blow_accumulator=0.0;
+	src->blow_loop_start=5000;
+	src->blow_loop_end=40000;
+
 }
 
 static void gst_jammo_slider_src_fixate (GstPad * pad, GstCaps * caps)
@@ -467,120 +538,241 @@ static gfloat freq_of_note[] = { 32.703201293945312, 34.647899627685547, 36.7080
 
 /*We roll over above frequence table (sorted) and when value from table is bigger
 than parameter, we check which one is nearer (which direction to round)*/
-static gfloat freq_rounding(gfloat slider_freq){
+static gfloat freq_rounding(gfloat freq){
  int i; //i starts from 1. All too low values are then rounded to table[0]
  for (i=1;i<NUMBER_OF_OCTAVES*NUMBER_OF_NOTES;i++){//roll over frequence table above (they are sorted)
-  if (freq_of_note[i]>slider_freq){                //continue until table has bigger than parameter
-    gfloat delta  =freq_of_note[i]-slider_freq;    //this is delta of parameter and bigger value from table
-    gfloat delta2 =slider_freq-freq_of_note[i-1];  //this is delta of parameter and smaller value from table
+  if (freq_of_note[i]>freq){                //continue until table has bigger than parameter
+    gfloat delta  =freq_of_note[i]-freq;    //this is delta of parameter and bigger value from table
+    gfloat delta2 =freq-freq_of_note[i-1];  //this is delta of parameter and smaller value from table
     return delta<delta2?freq_of_note[i]:freq_of_note[i-1]; //return value, which is nearer to parameter
   }
  }
  return 8000; //We could also return biggest value from table.
 }
 
+// a function for initializing parameters when starting to play a new note
+void process_slider_event_on(GstJammoSlider * src, float freq) {
+	src->state=TRUE;
+	src->freq=freq;
+	src->released=FALSE;
+	src->release_counter=0;
+	src->adsr_counter=0;
+	src->accumulator=0.0;
+	src->tremolo_phase=0.0;
+	src->rounded=FALSE;
+	src->rounding_counter=0;
+	// sliding to a new note is not wanted
+	src->freq_slow=freq;
+}
+
+void process_slider_event_off(GstJammoSlider * src, float freq) {
+	src->released=TRUE;
+	src->freq=freq;
+	src->rounding_counter=0;
+	src->rounded=FALSE;
+}
+
+void process_slider_event_motion(GstJammoSlider * src, float freq) {
+	src->freq=freq;
+	src->rounding_counter=0;
+	src->rounded=FALSE;
+}
+
+static void do_playback_process(GstJammoSlider * src){
+	/*GstClockTime now = gst_clock_get_time(gst_system_clock_obtain()); //time just now		
+	GstClockTime current = GST_CLOCK_DIFF(src->playback_starting_time,now);*/
+
+	JammoSliderEvent* event;
+	GstClockTime timestamp;
+	/*
+	timestamp is in nanoseconds = ns
+	e.g.:
+	6.081 683 703 s
+	6 081.683 703 ms
+	6 081 683.703 µs
+	6 081 683 703 ns
+	*/
+
+	event = g_list_nth_data(src->eventlist,src->playback_index_for_event);
+	if (event==NULL)
+		{
+			src->state=FALSE;
+			src->accumulator=0.0;
+			src->tremolo_phase=0.0;
+			src->eos_reached=TRUE;
+			return;
+		}
+	timestamp = event->timestamp;
+
+	while (src->time > timestamp){
+		printf("DEBUG: gstjammoslider: delayed: %llu ms, freq: %f, type: %d\n",(unsigned long long) (src->time - timestamp)/1000/1000, event->freq, event->type);
+		gfloat freq=event->freq;
+
+		if (freq >= MINFREQ && freq <= MAXFREQ) {
+			if (event->type==JAMMO_SLIDER_EVENT_ON ) {
+				process_slider_event_on(src, event->freq);
+			}
+
+			else if (event->type==JAMMO_SLIDER_EVENT_OFF) {
+				process_slider_event_off(src, event->freq);
+			}
+
+			else if (event->type==JAMMO_SLIDER_EVENT_MOTION) {
+				if (src->state==TRUE) {
+					process_slider_event_motion(src, event->freq);
+				}
+			}
+			else {
+				printf("a faulty event in list\n");
+				printf("freq: %f, type %d\n", freq, event->type);
+			}
+		}
+		else {
+				printf("a faulty event in list\n");
+				printf("freq: %f\n", freq);
+		}
+
+		// move to the next event
+		src->playback_index_for_event++;
+
+		// check if there are no more events
+		if (src->playback_index_for_event==g_list_length(src->eventlist)) {
+			src->last_event_processed=TRUE;
+			// release just in case some event-off is missing
+				src->released=TRUE;
+		}
+
+		// get next event from the list
+		event = g_list_nth_data(src->eventlist,src->playback_index_for_event);
+		if (event==NULL)
+		{
+			src->released=TRUE;
+			return;
+		}
+		// get the time of the event
+		timestamp = event->timestamp;
+	}
+
+}
 
 #define DEFINE_PROCESS(type,scale) \
 static void gst_jammo_slider_process_##type (GstJammoSlider * src, g##type * samples) {\
-	/* if volume is divided with number of playing tones instrument*/\
-	/*gets quieter when more simultaneous notes are played */ \
-\
-	/* this way overall volume can be adjusted according to number*/ \
-	/*of playing notes but without lowering the overall volume too much */ \
-\
-	/* however it is possible to get audible clipping this way. */\
-	/*one way to avoid this is not to scale samples to full volume, i.e.*/\
-	/*use smaller scale value than currently. in another words leaving some*/\
-	/*headroom to simultaneously playing samples */ \
-\
-	/* at the moment sampler works fine with flute, drum kit and ud.*/\
-	/*clipping is not too bad */\
   gint i;\
   g##type value=0;\
-  /*gdouble amp;*/ \
   gdouble step;\
-  gint counter=0;\
+\
+	if (!(src->recording)) { \
+		do_playback_process(src); \
+	} \
 \
   /*Process whole buffer*/ \
-  for (i = 0; i < src->generate_samples_per_buffer; i++) \
-      {\
-      /* Find what tone must be processed */ \
-      counter=0;\
+  for (i = 0; i < src->generate_samples_per_buffer; i++) {\
 \
-	  if ( src->slider_freq_slow < src->slider_freq &&  src->slider_freq - src->slider_freq_slow > 1.0){\
-	    src->slider_freq_slow+=0.02;\
-	    src->slider_rounding_counter=0;\
-	    }\
-	  else if ( src->slider_freq_slow > src->slider_freq && src->slider_freq_slow - src->slider_freq > 1.0 ){\
-	    src->slider_freq_slow-=0.02;\
-	    src->slider_rounding_counter=0;\
-	    }\
-	  else {\
-	    if (src->slider_state==TRUE)\
-	      src->slider_rounding_counter++;\
-	    }\
-\
-	  if (src->slider_rounding_counter>5000){\
-	  src->slider_freq=freq_rounding(src->slider_freq);\
-	/*  printf("do rounding\n");*/\
+		/* slide from current to target frequency */ \
+	  if ( src->freq_slow < src->freq &&  src->freq - src->freq_slow > 1.0){\
+			/* slide up */ \
+			if ((src->freq - src->freq_slow)*DISTANCEFACTOR > MINSLIDE) { \
+		    src->freq_slow+=(src->freq - src->freq_slow)*DISTANCEFACTOR; \
+			} \
+			else { \
+				src->freq_slow+=MINSLIDE; \
+			} \
+	    src->rounding_counter=0;\
 	  }\
+	  else if ( src->freq_slow > src->freq && src->freq_slow - src->freq > 1.0 ){\
+			/* slide down */ \
+			if ((src->freq_slow - src->freq)*DISTANCEFACTOR > MINSLIDE) { \
+	   	 src->freq_slow-=(src->freq_slow - src->freq)*DISTANCEFACTOR;\
+			} \
+			else { \
+				src->freq_slow-=MINSLIDE; \
+			} \
+	    src->rounding_counter=0;\
+		}\
+	  else {\
+			/* no need to slide */ \
+	    if (src->state==TRUE && src->rounded==FALSE) {\
+	      src->rounding_counter++;\
+			} \
+			/* round to nearest semitone if stayed in one frequency long enough */ \
+			if (src->rounding_counter==ROUNDINGCOUNT) { \
+		  	src->freq=freq_rounding(src->freq);\
+				src->rounded=TRUE; \
+	      src->rounding_counter++;\
+				printf("rounded\n"); \
+			} \
+		}\
 \
-/*printf("counter=%d, freq_slow=%f\n",src->slider_rounding_counter,src->slider_freq_slow);*/\
-	  step = M_PI_M2 * src->slider_freq_slow / src->samplerate;\
-	  src->slider_accumulator+= step;\
-	  if (src->slider_accumulator >= M_PI_M2)\
-			  src->slider_accumulator -= M_PI_M2;\
-	  /* to avoid clipping everything is multiplied with 0.4 (less volume).*/ \
-	  /*this is done to make synthesis about as loud as samples */ \
-	  if(src->slider_state){\
-            if (src->instrument==2)\
-            value = (g##type)( (0.7+fabs(0.3*sin(src->tremolo_phase))) *0.8* scale * (karplus(src) ));\
-            else\
-	    value = (g##type)( (0.7+fabs(0.3*sin(src->tremolo_phase))) *0.8* scale * (instruments[src->instrument](src->slider_accumulator) ));\
-}\
-	  else\
+\
+		/* calculate step size */\
+	  step = M_PI_M2 * src->freq_slow / src->samplerate;\
+	  src->accumulator+= step;\
+	  if (src->accumulator >= M_PI_M2)\
+			  src->accumulator -= M_PI_M2;\
+\
+	  /* generate sound */ \
+		if(src->state){\
+			if (src->instrument==2)\
+				value = (g##type)( (0.9+fabs(0.1*sin(src->tremolo_phase))) *0.8* scale * (karplus(src) ));\
+			else if (src->instrument==1) {\
+				/* add also blowing sound */ \
+				value = (g##type)( (0.9+fabs(0.1*sin(src->tremolo_phase))) *0.8* scale * (instruments[src->instrument](src->accumulator) )+ BLOW_VOLUME * src->blow_table[(gint)src->blow_accumulator] / (gfloat)G_MAXINT16 );\
+				src->blow_accumulator+=1; \
+			} \
+			else \
+				value = (g##type)( (0.9+fabs(0.1*sin(src->tremolo_phase))) *0.8* scale * (instruments[src->instrument](src->accumulator) ));\
+		}\
+	  else {\
 	    value=0;\
+		} \
 	  src->tremolo_phase += (M_PI*10/src->samplerate); \
 	  if (src->tremolo_phase>2*M_PI)\
 		    src->tremolo_phase-=2*M_PI;\
 \
-	  if (src->slider_state==TRUE) {\
+		if (src->blow_accumulator > src->blow_loop_end) { \
+			src->blow_accumulator=src->blow_loop_start; \
+			sf_seek(src->blow_file, src->blow_loop_start, SEEK_SET); \
+		} \
+\
+		/* if playing, process ADSR */ \
+		if (src->state==TRUE) {\
 	    if (!src->released) {\
 	      /* Attack */\
-	      if (src->adsr_counter<src->attack) { \
-		value*=src->adsr_counter/((gfloat)src->attack);\
-		src->release_multiplier=src->adsr_counter/((gfloat)src->attack);\
-		src->adsr_counter++;\
-		/*printf("attack\n");*/\
-	      }\
+				if (src->adsr_counter<src->attack) { \
+					value*=src->adsr_counter/((gfloat)src->attack);\
+					src->release_multiplier=src->adsr_counter/((gfloat)src->attack);\
+					src->adsr_counter++;\
+					/*printf("attack\n");*/\
+				}\
   \
 	      /* Decay */ \
-	      if (src->adsr_counter >= src->attack && src->adsr_counter < src->attack+src->decay) {\
-		value*=(1-src->sustain)*(1-(src->adsr_counter/((gfloat)(src->attack+src->decay))))+src->sustain;\
-		src->release_multiplier=(1-src->sustain)*(1-(src->adsr_counter/((gfloat)(src->attack+src->decay))))+src->sustain;\
-		src->adsr_counter++; \
-		/*printf("decay\n"); */\
+				if (src->adsr_counter >= src->attack && src->adsr_counter < src->attack+src->decay) {\
+					value*=(1-src->sustain)*(1-(src->adsr_counter/((gfloat)(src->attack+src->decay))))+src->sustain;\
+					src->release_multiplier=(1-src->sustain)*(1-(src->adsr_counter/((gfloat)(src->attack+src->decay))))+src->sustain;\
+					src->adsr_counter++; \
+					/*printf("decay\n");*/ \
 	      } \
   \
-	      /* Sustain */ \
-	      if (src->adsr_counter >= src->attack + src->decay) {\
-		value*=src->sustain; \
-		src->release_multiplier=src->sustain;\
-		/*printf("sustain\n");*/ \
+				/* Sustain */ \
+				if (src->adsr_counter >= src->attack + src->decay) {\
+					value*=src->sustain; \
+					src->release_multiplier=src->sustain;\
+					src->adsr_counter+=1; \
+					/*printf("sustain %f\n", src->sustain);*/ \
 	      }\
 	    }\
 \
-	    /* Release */ \
-	    if (src->released) {\
-	      value*=src->release_multiplier*(1-(src->release_counter/((gfloat)src->release)));\
-	      src->release_counter++;\
-	      if (src->release_counter>=src->release) \
-		src->slider_state=FALSE;\
-	     /* printf("release\n");*/ \
-	    } \
-	  }\
+			/* Release */ \
+			if (src->released) {\
+				value*=src->release_multiplier*(1-(src->release_counter/((gfloat)src->release)));\
+				src->release_counter++;\
+				if (src->release_counter >= src->release) \
+					src->state=FALSE;\
+				/*printf("release\n");*/ \
+			} \
+		}\
 	  samples[i]=value;\
-}\
+	} /* end process whole buffer */ \
 }
 
 DEFINE_PROCESS (int16, 32767.0);
@@ -600,9 +792,14 @@ static JammoSliderProcessFunc process_funcs[] = {
 static void gst_jammo_slider_get_times (GstBaseSrc * basesrc, GstBuffer * buffer,
     GstClockTime * start, GstClockTime * end)
 {
+
+	GstJammoSlider * src = GST_JAMMO_SLIDER(basesrc);
+
+	GstClockTime timestamp = GST_BUFFER_TIMESTAMP (buffer);
+	src->time=timestamp;
   /* for live sources, sync on the timestamp of the buffer */
   if (gst_base_src_is_live (basesrc)) {
-    GstClockTime timestamp = GST_BUFFER_TIMESTAMP (buffer);
+    
 
     if (GST_CLOCK_TIME_IS_VALID (timestamp)) {
       /* get duration to calculate end time */
@@ -622,7 +819,6 @@ static void gst_jammo_slider_get_times (GstBaseSrc * basesrc, GstBuffer * buffer
 static gboolean gst_jammo_slider_start (GstBaseSrc * basesrc)
 {
   GstJammoSlider *src = GST_JAMMO_SLIDER (basesrc);
-  int i;
 
   src->next_sample = 0;
   src->next_byte = 0;
@@ -631,9 +827,31 @@ static gboolean gst_jammo_slider_start (GstBaseSrc * basesrc)
   src->eos_reached = FALSE;
   src->tags_pushed = FALSE;
   src->tremolo_phase = 0;
-  for (i=0; i<NUMBER_OF_OCTAVES*NUMBER_OF_NOTES;i++)
-       src->accumulators[i] = 0;
+	src->adsr_counter=0;
+	src->accumulator=0;
 
+  //realtime recording
+  if (src->recording) {
+    printf("DEBUG:gstjammoslider: recording mode\n");
+		jammo_slider_event_free_glist(&(src->eventlist));
+    src->eventlist=NULL;
+		src->time_of_last_stored_event=0;
+  }
+  //playback
+  else  {
+    printf("DEBUG:gstjammoslider: not recording, playback\n");
+
+    src->playback_index_for_event=0;
+
+    //Test:
+   GList* list;
+   JammoSliderEvent* n;
+
+   for (list = src->eventlist; list; list = list->next) {
+      n = (list->data);
+      printf("TEST:gstjammoslider:In stack: %llu ms: freq:%f ,type: %d \n", (unsigned long long)n->timestamp/1000/1000,n->freq,n->type);
+   }
+  }
 
   return TRUE;
 }
@@ -723,10 +941,14 @@ static GstFlowReturn gst_jammo_slider_create (GstBaseSrc * basesrc,
 
   /* if no length was given, use our default length in samples otherwise convert
    * the length in bytes to samples. */
-  if (length == -1)
+  /*if (length == -1)
     samples = src->samples_per_buffer;
   else
-    samples = length / src->sample_size;
+    samples = length / src->sample_size;*/
+
+	// we need to use smaller buffer than requested
+	// otherwise there will be too much latency
+	samples = src->samples_per_buffer;
 
   /* if no offset was given, use our next logical byte */
   if (offset == -1)
@@ -821,7 +1043,7 @@ static void gst_jammo_slider_set_property (GObject * object, guint prop_id,
       src->release = g_value_get_int (value);
       break;
     case PROP_SUSTAIN:
-      src->sustain = g_value_get_double (value);
+      src->sustain = g_value_get_float (value);
       break;
     case PROP_VOLUME:
       src->volume = g_value_get_double (value);
@@ -839,26 +1061,115 @@ static void gst_jammo_slider_set_property (GObject * object, guint prop_id,
       src->can_activate_pull = g_value_get_boolean (value);
       break;
 
-    case PROP_SLIDER_FREQ:
-      src->slider_freq = g_value_get_float (value);
-      if (src->slider_first_freq)
-	{
-	src->slider_freq_slow=src->slider_freq;
-	src->slider_first_freq=FALSE;
-	}
-      break;
-    case PROP_SLIDER_STATE:
-     // src->slider_state = g_value_get_boolean (value);
-      if ((g_value_get_boolean (value))==TRUE) {
-	src->slider_state=TRUE;
-	src->released=FALSE;
-	src->release_counter=0;
-	src->adsr_counter=0;
-      }
-      else
-	src->released=TRUE;
-      break;
+		case PROP_REMOVE_EVENT:
+			;
+			// note that remove removes only one event
+			// to remove a note one has to remove on, off, and possible motion events
+			JammoSliderEvent* event = (JammoSliderEvent*)(g_value_get_pointer(value));
 
+			GList* list;
+			JammoSliderEvent* n;
+
+			//printf("Removing this: %llu ms: note:%d \n",(unsigned long long)time,note_);
+			for (list = src->eventlist; list; list = list->next) {
+				//printf(".\n");
+				n = (list->data);
+				//printf("Scanning stack: %llu ms: freq:%f ,type: %d \n", (unsigned long long)n->timestamp,n->freq, n->type);
+
+				// find and remove
+				if (n->type==event->type && n->freq==event->freq && n->timestamp==event->timestamp) {
+					printf("event removed\n");
+					src->eventlist = g_list_remove(src->eventlist,n);
+					g_free(n);
+				}
+
+			}
+
+		//printf("removing event done\n");
+			break;
+
+    case PROP_ADD_EVENT:
+			;
+			//JammoSliderEvent* event;
+			event = (JammoSliderEvent*)(g_value_get_pointer(value));
+			guint64 timestamp = event->timestamp;
+			gboolean store=TRUE;
+
+			//realtime. There are no timestamp in event, use current time
+			if (src->recording){
+				printf("realtime\n");
+				timestamp = src->time;
+
+				if (event->type==JAMMO_SLIDER_EVENT_ON) {
+					process_slider_event_on(src, event->freq);
+				}
+				else if (event->type==JAMMO_SLIDER_EVENT_OFF) {
+					process_slider_event_off(src, event->freq);
+				}
+				else if (event->type==JAMMO_SLIDER_EVENT_MOTION) {
+					if (src->state==TRUE) {
+						process_slider_event_motion(src, event->freq);
+						// check if event can be discarded
+						if (src->previous_event.type==JAMMO_SLIDER_EVENT_MOTION &&
+							timestamp - src->time_of_last_stored_event < DISCARDTIME) {
+							// find previous event from the list
+							GList * list;
+							JammoSliderEvent * temp_event;
+							for (list=src->eventlist;list;list=list->next) {
+								temp_event=list->data;
+								if (temp_event->type == src->previous_event.type &&
+									temp_event->freq == src->previous_event.freq &&
+									temp_event->timestamp == src->previous_event.timestamp) {
+										break;
+								}
+							}
+							if (list!=NULL) {
+								store=FALSE;
+							}
+						}
+					}
+					else {
+						store=FALSE;
+					}
+				}
+			}
+
+			else { //adding non-realtime (nothing special)
+				printf("non-realtime\n");
+				timestamp=event->timestamp;
+			}
+
+			if (store) {
+				jammo_slider_event_store_event_to_glist(&(src->eventlist) ,event->freq, event->type, timestamp);
+				printf("DEBUG: gstjammoslider: added event %llu ms: freq:'%f', type=%d\n",(unsigned long long)(timestamp/1000/1000),event->freq,event->type);
+				src->previous_event.type=event->type;
+				src->previous_event.freq=event->freq;
+				src->previous_event.timestamp=timestamp;
+				src->time_of_last_stored_event=timestamp;
+			}
+
+			break;
+    case PROP_CLEAR_EVENTS:
+      if (g_value_get_boolean (value)) {
+	      jammo_slider_event_free_glist(&(src->eventlist));
+	      src->eventlist=NULL;
+      }
+      break;
+    case PROP_RECORDING:
+      src->recording = g_value_get_boolean (value);
+      break;
+   case PROP_EVENTLIST:
+      //We got GList containing JammoSliderEvents, take all one-by-one and add to own internal eventlist
+			jammo_slider_event_free_glist(&(src->eventlist));
+			src->eventlist=NULL;
+      for (list = g_value_get_pointer (value); list; list = list->next) {
+        n = (list->data);
+        //printf("CASE PROP_EVENTLIST:gstjammosampler:Add: %llu ns: note:%d ,type: %s \n", (unsigned long long)n->timestamp,n->note, n->type==0? "start":"stop");
+				// we want to allocate memory for the event so that nothing else will delete events
+				// from samplers list
+				jammo_slider_event_store_event_to_glist(&(src->eventlist), n->freq, n->type, n->timestamp);
+        }
+			break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -887,7 +1198,7 @@ static void gst_jammo_slider_get_property (GObject * object, guint prop_id,
       g_value_set_int (value, src->release);
       break;
     case PROP_SUSTAIN:
-       g_value_set_double (value, src->sustain);
+       g_value_set_float (value, src->sustain);
       break;
     case PROP_VOLUME:
       g_value_set_double (value, src->volume);
@@ -904,12 +1215,12 @@ static void gst_jammo_slider_get_property (GObject * object, guint prop_id,
     case PROP_CAN_ACTIVATE_PULL:
       g_value_set_boolean (value, src->can_activate_pull);
       break;
-    case PROP_SLIDER_FREQ:
-       g_value_set_double (value, src->slider_freq);
-      break;
-    case PROP_SLIDER_STATE:
-      g_value_set_boolean (value,src->slider_state);
+    case PROP_RECORDING:
+      g_value_set_boolean (value, src->recording);
       break;
+		case PROP_EVENTLIST:
+			g_value_set_pointer (value, src->eventlist);
+			break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
