@@ -12,10 +12,11 @@
 #include "jammo-meam.h"
 #include "jammo-meam-private.h"
 
-G_DEFINE_TYPE(JammoEditingTrack, jammo_editing_track, JAMMO_TYPE_TRACK);
+G_DEFINE_TYPE(JammoEditingTrack, jammo_editing_track, JAMMO_TYPE_PLAYING_TRACK);
 
 enum {
 	PROP_0,
+	PROP_FIXED_DURATION
 };
 
 enum {
@@ -25,6 +26,7 @@ enum {
 };
 
 struct _JammoEditingTrackPrivate {
+	guint64 fixed_duration;
 	GstElement* bin;
 	GstElement* composition;
 	GList* samples;
@@ -40,6 +42,11 @@ JammoEditingTrack* jammo_editing_track_new() {
 	return JAMMO_EDITING_TRACK(g_object_new(JAMMO_TYPE_EDITING_TRACK, NULL));
 }
 
+JammoEditingTrack* jammo_editing_track_new_fixed_duration(guint64 duration) {
+
+	return JAMMO_EDITING_TRACK(g_object_new(JAMMO_TYPE_EDITING_TRACK, "fixed-duration", duration, NULL));
+}
+
 void jammo_editing_track_add_sample(JammoEditingTrack* editing_track, JammoSample* sample, guint64 position) {
 	JammoEditingTrack* previous_track;
 	GList* list;
@@ -132,6 +139,11 @@ guint64 jammo_track_get_sample_position(JammoEditingTrack* editing_track, JammoS
 
 }
 
+guint64 jammo_editing_track_get_fixed_duration(JammoEditingTrack* track) {
+
+	return track->priv->fixed_duration;
+}
+
 static guint64 jammo_editing_track_get_duration(JammoTrack* track) {
 	guint64 duration = 0;
 	JammoEditingTrack* editing_track;
@@ -139,20 +151,25 @@ static guint64 jammo_editing_track_get_duration(JammoTrack* track) {
 	guint64 d;
 	
 	editing_track = JAMMO_EDITING_TRACK(track);
-	for (list = editing_track->priv->samples; list; list = list->next) {
-		if ((d = jammo_sample_get_duration(JAMMO_SAMPLE(list->data))) == JAMMO_DURATION_INVALID) {
-			duration = JAMMO_DURATION_INVALID;
-			break;
+	
+	if (editing_track->priv->fixed_duration != JAMMO_DURATION_INVALID) {
+		duration = editing_track->priv->fixed_duration;
+	} else {
+		for (list = editing_track->priv->samples; list; list = list->next) {
+			if ((d = jammo_sample_get_duration(JAMMO_SAMPLE(list->data))) == JAMMO_DURATION_INVALID) {
+				duration = JAMMO_DURATION_INVALID;
+				break;
+			}
+			duration += d;
 		}
-		duration += d;
 	}
 
 	return duration;
 }
 
-static GstElement* jammo_editing_track_get_element(JammoTrack* track) {
+static GstElement* jammo_editing_track_get_playing_element(JammoPlayingTrack* playing_track) {
 
-	return JAMMO_EDITING_TRACK(track)->priv->bin;
+	return JAMMO_EDITING_TRACK(playing_track)->priv->bin;
 }
 
 static void jammo_editing_track_set_property(GObject* object, guint prop_id, const GValue* value, GParamSpec* pspec) {
@@ -161,6 +178,9 @@ static void jammo_editing_track_set_property(GObject* object, guint prop_id, con
 	track = JAMMO_EDITING_TRACK(object);
 
 	switch (prop_id) {
+		case PROP_FIXED_DURATION:
+			track->priv->fixed_duration = g_value_get_uint64(value);
+			break;
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
 			break;
@@ -168,17 +188,52 @@ static void jammo_editing_track_set_property(GObject* object, guint prop_id, con
 }
 
 static void jammo_editing_track_get_property(GObject* object, guint prop_id, GValue* value, GParamSpec* pspec) {
-        JammoEditingTrack* editing_track;
+        JammoEditingTrack* track;
 
-	editing_track = JAMMO_EDITING_TRACK(object);
+	track = JAMMO_EDITING_TRACK(object);
 
         switch (prop_id) {
+		case PROP_FIXED_DURATION:
+			g_value_set_uint64(value, track->priv->fixed_duration);
+			break;
 	        default:
 		        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
 		        break;
         }
 }
 
+static void jammo_editing_track_constructed(GObject* object) {
+	JammoEditingTrack* editing_track;
+	gint64 duration;
+	GstElement* source;
+	GstElement* element;
+	
+	editing_track = JAMMO_EDITING_TRACK(object);
+	
+	if (editing_track->priv->fixed_duration != JAMMO_DURATION_INVALID) {
+		duration = editing_track->priv->fixed_duration;
+	} else {
+		duration = 0.1 * GST_SECOND;
+	}
+
+	source = gst_element_factory_make("gnlsource", NULL);
+	g_object_set(source,
+	             "priority", 1,
+		     "start", 0 * GST_SECOND,
+		     "duration", duration,
+		     "media-start", 0 * GST_SECOND,
+		     "media-duration", duration,
+	             NULL);
+	element = gst_element_factory_make("audiotestsrc", NULL);
+	g_object_set(element,
+	             "wave", 4, /* GST_AUDIO_TEST_SRC_WAVE_SILENCE */
+	             NULL);
+	gst_bin_add(GST_BIN(source), element);
+	gst_bin_add(GST_BIN(editing_track->priv->composition), source);
+
+	G_OBJECT_CLASS(jammo_editing_track_parent_class)->constructed(object);
+}
+
 static void jammo_editing_track_finalize(GObject* object) {
 	G_OBJECT_CLASS(jammo_editing_track_parent_class)->finalize(object);
 }
@@ -198,14 +253,28 @@ static void jammo_editing_track_dispose(GObject* object) {
 static void jammo_editing_track_class_init(JammoEditingTrackClass* editing_track_class) {
 	GObjectClass* gobject_class = G_OBJECT_CLASS(editing_track_class);
 	JammoTrackClass* track_class = JAMMO_TRACK_CLASS(editing_track_class);
+	JammoPlayingTrackClass* playing_track_class = JAMMO_PLAYING_TRACK_CLASS(editing_track_class);
 
 	track_class->get_duration = jammo_editing_track_get_duration;
-	track_class->get_element = jammo_editing_track_get_element;
+	playing_track_class->get_playing_element = jammo_editing_track_get_playing_element;
+	gobject_class->constructed = jammo_editing_track_constructed;
 	gobject_class->finalize = jammo_editing_track_finalize;
 	gobject_class->dispose = jammo_editing_track_dispose;
 	gobject_class->set_property = jammo_editing_track_set_property;
 	gobject_class->get_property = jammo_editing_track_get_property;
 
+	/**
+	 * JammoSequencer:fixed-duration:
+	 *
+	 * The fixed duration of the track or JAMMO_DURATION_INVALID if the track duration is dynamic.
+	 */
+	g_object_class_install_property(gobject_class, PROP_FIXED_DURATION,
+	                                g_param_spec_uint64("fixed-duration",
+	                                "Fixed duration",
+	                                "The fixed duration of the track or JAMMO_DURATION_INVALID if the track duration is dynamic",
+	                                0, G_MAXUINT64, JAMMO_DURATION_INVALID,
+	                                G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK |G_PARAM_STATIC_BLURB));
+
 	/**
 	 * JammoEditingTrack:sample-added:
 	 * @editing_track: the object which received the signal
@@ -272,26 +341,11 @@ static void jammo_editing_track_init(JammoEditingTrack* editing_track) {
 		     "duration", 1000 * GST_SECOND,
 		     "media-start", 0 * GST_SECOND,
 		     "media-duration", 1000 * GST_SECOND,
-		     NULL);	
+	             NULL);
 	element = gst_element_factory_make("audiotestsrc", NULL);
 	g_object_set(element,
 	             "wave", 4, /* GST_AUDIO_TEST_SRC_WAVE_SILENCE */
-		     NULL);
-	gst_bin_add(GST_BIN(source), element);
-	gst_bin_add(GST_BIN(editing_track->priv->composition), source);
-
-	source = gst_element_factory_make("gnlsource", NULL);
-	g_object_set(source,
-	             "priority", 1,
-		     "start", 0 * GST_SECOND,
-		     "duration", 1 * GST_SECOND,
-		     "media-start", 0 * GST_SECOND,
-		     "media-duration", 1 * GST_SECOND,
-		     NULL);	
-	element = gst_element_factory_make("audiotestsrc", NULL);
-	g_object_set(element,
-	             "wave", 4, /* GST_AUDIO_TEST_SRC_WAVE_SILENCE */
-		     NULL);
+	             NULL);
 	gst_bin_add(GST_BIN(source), element);
 	gst_bin_add(GST_BIN(editing_track->priv->composition), source);
 }
