@@ -13,6 +13,7 @@
 #include "gst-jammo-plugins/pitchdetect-plugin/gstjammopitchdetect.h"
 #include "gst-jammo-plugins/sampler-plugin/gstjammosampler.h"
 #include "gst-jammo-plugins/slider-plugin/gstjammoslider.h"
+#include "gst-jammo-plugins/metronome-plugin/gstjammometronome.h"
 
 #include <gst/gst.h>
 #include <gst/controller/gstcontroller.h>
@@ -26,6 +27,7 @@ void jammo_meam_init(int* argc, char** argv[]) {
 	gst_element_register(NULL, "pitchdetect", 0, GST_TYPE_PITCHDETECT);
 	gst_element_register(NULL, "jammosampler", 0, GST_TYPE_JAMMO_SAMPLER);
 	gst_element_register(NULL, "jammoslider", 0, GST_TYPE_JAMMO_SLIDER);
+	gst_element_register(NULL, "jammometronome", 0, GST_TYPE_JAMMO_METRONOME);
 }
 
 void jammo_meam_cleanup(void) {
